/******************************************************************************
  Target Script for Atmel AT91SAM7SE.

  Copyright (c) 2007 Rowley Associates Limited.
  Partial Copyright (c) 2009, Michael Fischer

  This file may be distributed under the terms of the License Agreement
  provided with this software.

  THIS FILE IS PROVIDED AS IS WITH NO WARRANTY OF ANY KIND, INCLUDING THE
  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 ******************************************************************************/

function SDRAMInit()
{
  /* Enable SDRAM control at PIO A. */
  TargetInterface.pokeWord(0xfffff474, 0x3f800000); /* PIO_BSR_OFF */
  TargetInterface.pokeWord(0xfffff404, 0x3f800000); /* PIO_PDR_OFF */

  /* Enable address bus (A0, A2-A11, A13-A17) at PIO B */
  TargetInterface.pokeWord(0xfffff674, 0x0003effd); /* PIO_BSR_OFF */
  TargetInterface.pokeWord(0xfffff604, 0x0003effd); /* PIO_PDR_OFF */

  /* Enable 16 bit data bus at PIO C */
  TargetInterface.pokeWord(0xfffff870, 0x0000ffff); /* PIO_ASR_OFF */
  TargetInterface.pokeWord(0xfffff804, 0x0000ffff); /* PIO_PDR_OFF */

  /* Enable SDRAM chip select */
  TargetInterface.pokeWord(0xffffff80, 0x00000002); /* EBI_CSA_OFF */

  /* Set SDRAM characteristics in configuration register. */
  /* Hard coded values for MT48LC32M16A2 with 48MHz CPU.  */
  TargetInterface.pokeWord(0xffffffb8, 0x2192215a); /* SDRAMC_CR_OFF */
  TargetInterface.delay(10);

  /* Issue 16 bit SDRAM command: NOP  */
  TargetInterface.pokeWord(0xffffffb0, 0x00000011); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 

  /* Issue 16 bit SDRAM command: Precharge all */
  TargetInterface.pokeWord(0xffffffb0, 0x00000012); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 

  /* Issue 8 auto-refresh cycles */
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 
  TargetInterface.pokeWord(0xffffffb0, 0x00000014); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000); 

  /* Issue 16 bit SDRAM command: Set mode register */
  TargetInterface.pokeWord(0xffffffb0, 0x00000013); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000014, 0xcafedede);

  /* Set refresh rate count */
  TargetInterface.pokeWord(0xffffffb4, 384);        /* SDRAMC_TR_OFF */

  /* Issue 16 bit SDRAM command: Normal mode */
  TargetInterface.pokeWord(0xffffffb0, 0x00000010); /* SDRAMC_MR_OFF */
  TargetInterface.pokeWord(0x20000000, 0x00000000);
}

function Reset()
{
  /* Reset and stop target */
  TargetInterface.resetAndStop(1);

  CKGR_MOR_VAL = 0x00004001;
  CKGR_PLLR_VAL = 0x1048100E; /* Set up the PLL (MCK, UDPCK = 18.432MHz * 73 / 14 / 2 = 48.05486MHz) */
  CKGR_MCKR_VAL1 = 0x00000004;
  CKGR_MCKR_VAL2 = 0x00000007;

  if (TargetInterface.setMaximumJTAGFrequency)
    {
      /* Configure Clock (post 1.6 version) */
      TargetInterface.setMaximumJTAGFrequency(32768);
      TargetInterface.pokeWord(0xFFFFFC20, CKGR_MOR_VAL);
      TargetInterface.pokeWord(0xFFFFFC2C, CKGR_PLLR_VAL);
      TargetInterface.pokeWord(0xFFFFFC30, CKGR_MCKR_VAL1);
      TargetInterface.pokeWord(0xFFFFFC30, CKGR_MCKR_VAL2);
      TargetInterface.setMaximumJTAGFrequency(10000000);
    }
  else
    {
      /* Configure Clock (1.6 version) */
      TargetInterface.pokeWord(0xFFFFFC20, CKGR_MOR_VAL);
      TargetInterface.delay(10);
      TargetInterface.pokeWord(0xFFFFFC2C, CKGR_PLLR_VAL);
      TargetInterface.delay(10);
      TargetInterface.pokeWord(0xFFFFFC30, CKGR_MCKR_VAL1);
      TargetInterface.delay(10);
      TargetInterface.pokeWord(0xFFFFFC30, CKGR_MCKR_VAL2);
      TargetInterface.delay(10);
    }

  /* Setup internal FLASH wait states */
  TargetInterface.pokeWord(0xFFFFFF60, 0x00000200);
  if (((TargetInterface.peekWord(0xFFFFF240) >> 8) & 0xF) >= 0xA)
    TargetInterface.pokeWord(0xFFFFFF70, 0x00000200);
}

function RAMReset()
{
  Reset();
  /* Remap SRAM to 0x00000000 */
  resetVector = TargetInterface.peekWord(0x00000000);
  ramResetVector = TargetInterface.peekWord(0x00200000);
  remap = resetVector != ramResetVector;
  if (!remap)
    {
      TargetInterface.pokeWord(0x00200000, resetVector ^ 0xFFFFFFFF);
      if (TargetInterface.peekWord(0x00000000) == resetVector)
        remap = 1;
      TargetInterface.pokeWord(0x00200000, ramResetVector);
    }
  if (remap)
    TargetInterface.pokeWord(0xFFFFFF00, 1); // MC_RCR 

  SDRAMInit();
}

function FLASHReset()
{
  Reset();
}

